﻿using System.IO;
using System.Web;
using Kendo.Web.Models.FileBrowser;
using Kendo.Web.Services;
using Orchard;
using Orchard.Localization;
using System;
using System.Linq;
using System.Web.Mvc;

namespace Kendo.Web.Controllers
{
    public class FileBrowserController : Controller  
    {
        private readonly IKendoImageBrowser kendoImageBrowser;
        private readonly IThumbnailCreator thumbnailCreator;

        public FileBrowserController(
            IOrchardServices orchardServices,
            IKendoImageBrowser kendoImageBrowser,
            IThumbnailCreator thumbnailCreator)
        {
            this.thumbnailCreator = thumbnailCreator;
            this.kendoImageBrowser = kendoImageBrowser;
            this.Services = orchardServices;
            this.T = NullLocalizer.Instance;
        }

        public IOrchardServices Services { get; private set; }
        
        public Localizer T { get; set; }

        private string StartPath() 
        {
            return "~/";
        }

        public virtual ActionResult Read(string path, string type)
        {
            if (!this.Services.Authorizer.Authorize(Permissions.AccessMediaPath)) {
                return new HttpStatusCodeResult(403, "Forbidden");
            }

            path = string.IsNullOrWhiteSpace(path) ? null : path;

            if (!kendoImageBrowser.DirectoryExists(path)) 
            {
                return new HttpStatusCodeResult(404, "File or Folder not found");
            }

            var result = kendoImageBrowser.GetContent(path)
                .Select(f => new
                {
                    name = f.Name,
                    type = f.Type == EntryType.File ? "f" : "d",
                    size = f.Size
                }).Where(e => string.IsNullOrWhiteSpace(type) ? true : e.type.Equals(type));

            return Json(result, JsonRequestBehavior.AllowGet);
        }

        [OutputCache(Duration = 3600, VaryByParam = "path")]
        public virtual ActionResult Thumbnail(string path) 
        {
            if (!this.Services.Authorizer.Authorize(Permissions.AccessMediaPath))
            {
                return new HttpStatusCodeResult(403, "Forbidden");
            }

            if (!kendoImageBrowser.FileExists(path))
            {
                return new HttpStatusCodeResult(404, "File not found");
            }

            var file = this.kendoImageBrowser.FilePath(path);
            //var fileName = Path.GetFileName(path);
            //Response.AddFileDependency(filename);

            using (var fileStream = file.OpenRead())
            {
                var desiredSize = new ImageSize
                {
                    Width = 80,
                    Height = 80
                };

                string contentType = "image/png";
                var thumbData = thumbnailCreator.Create(fileStream, desiredSize, contentType);
                return File(thumbData, contentType);
            }
        }

        [HttpPost]
        public ActionResult Destroy(string path, string name, string type) 
        {
            if (!string.IsNullOrEmpty(name) && !string.IsNullOrEmpty(type))
            {

                if (type.ToLowerInvariant() == "f")
                {
                    kendoImageBrowser.DeleteFile(path, name);
                }
                else
                {
                    kendoImageBrowser.DeleteDirectory(path, name);
                }

                return Json(null);
            }

            return new HttpStatusCodeResult(404, "File Not Found"); 
        }

        public ActionResult Create(string path, FileBrowserEntry entry)
        {
            if (!this.Services.Authorizer.Authorize(Permissions.AccessMediaPath))
            {
                return new HttpStatusCodeResult(403, "Forbidden");
            }

            var name = entry.Name;

            if (!string.IsNullOrEmpty(name))
            {
                kendoImageBrowser.CreateDirectory(path, name);
            }

            return Json(null);
        }

        [HttpPost]
        public ActionResult Upload(string path, HttpPostedFileBase file) 
        {
            if (!this.Services.Authorizer.Authorize(Permissions.AccessMediaPath))
            {
                return new HttpStatusCodeResult(403, "Forbidden");
            }

            var fileName = Path.GetFileName(file.FileName);
            var files = this.kendoImageBrowser.GetContent(path).Where(e=> e.Type == EntryType.File);
            var foundFile = files.FirstOrDefault(e=> e.Name.Equals(file.FileName, StringComparison.InvariantCultureIgnoreCase));

            if (foundFile != null)
            {
                this.kendoImageBrowser.DeleteFile(path, file.FileName);
            }
            //if (Path.GetExtension(file.FileName).EndsWith("zip")) {
            //    string pathFolderName = Path.GetFileNameWithoutExtension(file.FileName);
            //    string newPath = Path.Combine(path, pathFolderName);
            //    this.kendoImageBrowser.CreateDirectory(path, pathFolderName);

            //    this.kendoImageBrowser.UploadFile(newPath, file);
            //    var folder = this.kendoImageBrowser.FolderPath(newPath);
            //    return Json(new {
            //        size = folder.Size,
            //        name = pathFolderName,
            //        type = "d"
            //    });
            //}

            this.kendoImageBrowser.UploadFile(path, file);
            
            return Json(new {
                size = file.ContentLength,
                name = fileName,
                type = "f"
            }, "text,plain");
        }

    }
}
