﻿using Kendo.Web.Services.Scheduler;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Web.Mvc;
using Orchard.ContentManagement;
using Kendo.Web.ViewModels;
using Kendo.Web.Models;
using Orchard;
using Orchard.Settings;
using Orchard.DisplayManagement;
using Orchard.Environment.Extensions;

namespace Kendo.Web.Controllers
{
    [OrchardFeature("Kendo.UI.Scheduler")]
    public partial class SchedulerTaskController : Controller
    {
        private readonly ITaskSchedulerService taskSchedulerService;

        private readonly ISiteService siteService;

        /// <summary>
        /// Initializes a new instance of the <see cref="SchedulerTaskController" /> class.
        /// </summary>
        /// <param name="taskSchedulerService">The task scheduler service.</param>
        /// <param name="services">The services.</param>
        /// <param name="siteService">The site service.</param>
        /// <param name="shapeFactory">The shape factory.</param>
        public SchedulerTaskController(
            ITaskSchedulerService taskSchedulerService,
            IOrchardServices services,
            ISiteService siteService,
            IShapeFactory shapeFactory) 
        {
            this.Shape = shapeFactory;
            this.siteService = siteService;
            this.Services = services;
            this.taskSchedulerService = taskSchedulerService;
        }

        public IOrchardServices Services { get; private set; }

        public dynamic Shape { get; private set; }

        //public ActionResult Index(AdminIndexOptions options, PagerParameters pagerParameters) 
        //{
        //    var pager = new Pager(siteService.GetSiteSettings(), pagerParameters);
        //    var queries = Services.ContentManager
        //        .Query(VersionOptions.Published)
        //        .ForType("Query")
        //        .Join<TitlePartRecord>().OrderBy(e=> e.Title);

        //    var results = queries
        //        .Slice(pager.GetStartIndex(), pager.PageSize)
        //        .ToList();

        //    var pagerShape = Shape.Pager(pager).TotalItemCount(queries.Count());

        //    var model = new AdminIndexViewModel
        //    {
        //        Queries = results.Select(x => new QueryEntry
        //        {
        //            Query = x.As<QueryPart>().Record,
        //            QueryId = x.Id,
        //            Name = x.As<QueryPart>().Name
        //        }).ToList(),
        //        Options = options,
        //        Pager = pagerShape
        //    };

        //    var routeData = new RouteData();
        //    routeData.Values.Add("Options.Filter", options.Filter);
        //    routeData.Values.Add("Options.Search", options.Search);
        //    routeData.Values.Add("Options.Order", options.Order);

        //    pagerShape.RouteData(routeData);

        //    return View();
        //}

        /// <summary>
        /// Read items if the user has the permission
        /// </summary>
        /// <param name="queryId">A projection query id</param>
        /// <returns></returns>
        public virtual ActionResult Read(int queryId) 
        {
            if (!Services.Authorizer.Authorize(PermissionsScheduler.ReadTasks))
            {
                return new HttpStatusCodeResult(System.Net.HttpStatusCode.Forbidden);
            }

            var data = taskSchedulerService.GetContent(queryId);
            var dataModels = data.Select(e => e.As<IKendoScheduleTask>().ToViewModel()).ToList();

            var viewModel = new KendoGridResult()
            {
                Data = dataModels,
                Total = dataModels.Count,
                Errors = null
            };

            return Json(viewModel, JsonRequestBehavior.AllowGet);
        }

        /// <summary>
        /// Creates the specified task if the user has permission.
        /// </summary>
        /// <param name="viewModel">The view model.</param>
        /// <returns></returns>
        public virtual ActionResult Create(SchedulerTaskViewModel viewModel)
        {
            if (!Services.Authorizer.Authorize(PermissionsScheduler.CreateTask)) 
            {
                return new HttpStatusCodeResult(System.Net.HttpStatusCode.Forbidden);
            }

            if (ModelState.IsValid) 
            {
                taskSchedulerService.Create(viewModel);
            }

            var data = new KendoGridResult()
            {
                Data = new[] { viewModel },
                Total = 1,
                Errors = Errors().FirstOrDefault()
            };
            return Json(data);
        }

        /// <summary>
        /// Updates the specified task if the user has permission.
        /// </summary>
        /// <param name="viewModel">The view model.</param>
        /// <returns></returns>
        public virtual ActionResult Update(SchedulerTaskViewModel viewModel) 
        {
            if (!Services.Authorizer.Authorize(PermissionsScheduler.EditTask))
            {
                return new HttpStatusCodeResult(System.Net.HttpStatusCode.Forbidden);
            }

            if (ModelState.IsValid) 
            {
                taskSchedulerService.Update(viewModel);
            }

            var data = new KendoGridResult()
            {
                Data = new[] { viewModel },
                Total = 1,
                Errors = Errors().FirstOrDefault()
            };

            return Json(data);
        }

        /// <summary>
        /// Destroys the specified task if the user has permission.
        /// </summary>
        /// <param name="viewModel">The view model.</param>
        /// <returns></returns>
        public virtual ActionResult Destroy(SchedulerTaskViewModel viewModel)
        {
            if (!Services.Authorizer.Authorize(PermissionsScheduler.DestroyTask))
            {
                return new HttpStatusCodeResult(System.Net.HttpStatusCode.Forbidden);
            }

            if (ModelState.IsValid) 
            { 
                taskSchedulerService.Delete(viewModel.TaskId);
            }

            var data = new KendoGridResult()
            {
                Data = new[] { viewModel },
                Total = 1,
                Errors = Errors().FirstOrDefault()
            };

            return Json(data);
            //return new HttpStatusCodeResult(200);
        }

        public IEnumerable<string> Errors() 
        {
            if(ModelState.IsValid)
                return Enumerable.Empty<string>();

            var errors = this.ModelState.Values.SelectMany(e => e.Errors).Select(e=> e.ErrorMessage);
         
            return errors;
        }
    }
}