﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Web.Mvc;
using System.Xml;
using System.Xml.Linq;
using System.Xml.XPath;

namespace Kendo.Web.Extensions
{
    public static class HtmlExtensions
    {
        /// <summary>
        /// Grab a Summary from the text. A small adaptation from http://stackoverflow.com/questions/1714764/c-sharp-truncate-html-safely-for-article-summary
        /// It runs through the html to build a 'summary' till the character count lengthAvailable is meet. Building the correct html as it goes.
        /// Or runs through and grabs elements (ie div) with the class 'summary' and builds the output from that.
        /// </summary>
        /// <param name="helper">The helper.</param>
        /// <param name="html">The HTML.</param>
        /// <param name="lengthAvailable">The length available.</param>
        /// <returns></returns>
        public static MvcHtmlString SummaryText(this HtmlHelper helper, string html, int lengthAvailable = 500) 
        {
            // declare nbsp as xml entity and its value is " " in this case.
            string declareEntity = "<!DOCTYPE xml [<!ENTITY nbsp \" \">]>";

            var xdoc = new XDocument(
                new XDeclaration("1.0", "utf-8", null),
                XElement.Parse(declareEntity + "<div>" + html + "</div>")
            );

            string summary = xdoc.ExtractSummary();

            if (!string.IsNullOrWhiteSpace(summary))
                return new MvcHtmlString(summary);

            XmlDocument xml = xdoc.ToXmlDocument(); 
            // create a navigator, this is our primary tool
            XPathNavigator navigator = xml.CreateNavigator();
            XPathNavigator breakPoint = null;

            string lastText = "";
 
            // find the text node we need:
            while (navigator.MoveToFollowing(XPathNodeType.Text))
            {
                lastText = navigator.Value.Substring(0, Math.Min(lengthAvailable, navigator.Value.Length));
                lengthAvailable -= navigator.Value.Length;

                if (lengthAvailable <= 0)
                {
                    // truncate the last text. Here goes your "search word boundary" code:
                    navigator.SetValue(lastText);
                    breakPoint = navigator.Clone();
                    break;
                }
            }

            // first remove text nodes, because Microsoft unfortunately merges them without asking
            while (navigator.MoveToFollowing(XPathNodeType.Text))
                if (navigator.ComparePosition(breakPoint) == XmlNodeOrder.After)
                    navigator.DeleteSelf();   // moves to parent

            // then move the rest
            navigator.MoveTo(breakPoint);
            while (navigator.MoveToFollowing(XPathNodeType.Element))
                if (navigator.ComparePosition(breakPoint) == XmlNodeOrder.After)
                    navigator.DeleteSelf();   // moves to parent

            // then remove *all* empty nodes to clean up (not necessary): 
            // TODO, add empty elements like <br />, <img /> as exclusion
            navigator.MoveToRoot();
            while (navigator.MoveToFollowing(XPathNodeType.Element))
                while (!navigator.HasChildren && (navigator.Value ?? "").Trim() == "")
                    navigator.DeleteSelf();  // moves to parent

            navigator.MoveToRoot();

            return new MvcHtmlString(navigator.InnerXml);
        }

        private static string ExtractSummary(this XDocument xDoc) 
        {
            var summaryItems = xDoc.Root
                .Elements("div")
                .Where(element => element.Attribute("class").Value.Equals("summary")).ToList();

            StringBuilder sb = new StringBuilder();

            foreach (var element in summaryItems) 
            {
                sb.Append(element.ToString());
            }

            return sb.ToString();
        }

        private static XDocument ToXDocument(this XmlDocument xmlDocument)
        {
            using (var nodeReader = new XmlNodeReader(xmlDocument))
            {
                nodeReader.MoveToContent();
                return XDocument.Load(nodeReader);
            }
        }

        private static XmlDocument ToXmlDocument(this XDocument xDocument)
        {
            var xmlDocument = new XmlDocument();
            using (var xmlReader = xDocument.CreateReader())
            {
                xmlDocument.Load(xmlReader);
            }
            return xmlDocument;
        }
    }
}
