﻿using System;
using System.Linq;
using Orchard.ContentManagement.MetaData;
using Orchard.Core.Contents.Extensions;
using Orchard.Data.Migration.Schema;

namespace Kendo.Web.MigrationTasks
{
    public static class Scheduler
    {
        /// <summary>
        /// Drops the scheduler task part table.
        /// </summary>
        /// <param name="schemeBuilder">The scheme builder.</param>
        internal static void DropSchedulerTaskPartTable(SchemaBuilder schemeBuilder) 
        {
            var tableName = typeof(Models.KendoTaskPartRecord).Name;

            schemeBuilder.DropTable(tableName);
        }

        /// <summary>
        /// Creates the scheduler task part table.
        /// </summary>
        /// <param name="schemaBuilder">The schema builder.</param>
        internal static void CreateSchedulerTaskPartTable(SchemaBuilder schemaBuilder)
        {
            var tableName = typeof(Models.KendoTaskPartRecord).Name;

            schemaBuilder.CreateTable(tableName, table => 
                table
                    .ContentPartRecord()
                    .Column<string>("Description", c => c.Unlimited())
                    .Column<DateTime>("StartUtc")
                    .Column<DateTime>("EndUtc")
                    .Column<bool>("IsAllDay")
                    .Column<string>("RecurrenceRule", c=> c.Nullable())
                    .Column<int>("RecurrenceId", c=> c.Nullable())
                    .Column<string>("RecurrenceException", c=> c.Nullable())
                );
        }

        /// <summary>
        /// Creates the type of the content.
        /// </summary>
        /// <param name="contentDefinitionManager">The content definition manager.</param>
        internal static void CreateContentType(IContentDefinitionManager contentDefinitionManager)
        {
            var scheduleTaskPart = typeof(Models.KendoTaskPart).Name;
 
            contentDefinitionManager.AlterTypeDefinition(DefinitionContentType.KendoSchedulerTaskType, alter =>
                alter
                    .DisplayedAs("Scheduler Task")
                    .WithPart("CommonPart")
                    .WithPart("TitlePart")
                    .WithPart("AutoroutePart")
                    .WithPart(scheduleTaskPart)
            );
        }

        /// <summary>
        /// Adds the scheduler task settings.
        /// </summary>
        /// <param name="contentDefinitionManager">The content definition manager.</param>
        internal static void AddSchedulerTaskSettings(IContentDefinitionManager contentDefinitionManager)
        {
            string creatable = "ContentTypeSettings.Creatable";
            string draftable = "ContentTypeSettings.Draftable";

            contentDefinitionManager.AlterTypeDefinition(DefinitionContentType.KendoSchedulerTaskType, alter => 
                alter
                    .WithSetting(creatable, bool.TrueString)
                    .WithSetting(draftable, bool.TrueString)
            );
        }

        /// <summary>
        /// Adds autoroute settings for the scheduler.
        /// </summary>
        /// <param name="contentDefinitionManager">The content definition manager.</param>
        internal static void AddSchedulerAutorouteSettings(IContentDefinitionManager contentDefinitionManager)
        {
            contentDefinitionManager.AlterTypeDefinition(DefinitionContentType.KendoSchedulerTaskType, alter =>
                alter.WithPart("AutoroutePart", part =>
                    part
                        .WithSetting("AutorouteSettings.AllowCustomPattern", bool.TrueString)
                        .WithSetting("AutorouteSettings.AutomaticAdjustmentOnEdit", bool.TrueString)
                        .WithSetting("AutorouteSettings.PerItemConfiguration", bool.FalseString)
                        .WithSetting("AutorouteSettings.PatternDefinitions", "[{Name:'scheduler/Title', Pattern: 'scheduler/{Content.Slug}', Description: 'scheduler route'}]")
                        .WithSetting("AutorouteSettings.DefaultPatternIndex", "0")
                )
            );
        }

        /// <summary>
        /// Make the kendo task part attachable
        /// </summary>
        /// <param name="contentDefinitionManager">The content definition manager.</param>
        internal static void AttachablePart(IContentDefinitionManager contentDefinitionManager)
        {
            var scheduleTaskPart = typeof(Models.KendoTaskPart).Name;

            contentDefinitionManager.AlterPartDefinition(scheduleTaskPart, alter => alter.Attachable());
        }

        
    }
}