using Kendo.Web.Models.Scheduler;
using Orchard.ContentManagement;
using Orchard.Data;
using Orchard.Forms.Services;
using Orchard.Localization;
using Orchard.Projections.Descriptors;
using Orchard.Projections.Descriptors.Filter;
using Orchard.Projections.Models;
using Orchard.Projections.Services;
using System;
using System.Collections.Generic;
using System.Linq;
using Orchard.Tokens;
using Orchard.Projections.Descriptors.SortCriterion;
using Orchard.Core.Title.Models;
using Kendo.Web.Models;

namespace Kendo.Web.Services.Scheduler
{
    public class TaskSchedulerService : ITaskSchedulerService
    {
        private readonly IRepository<QueryPartRecord> queryRepository;
        private readonly IContentManager contentManager;
        private readonly IProjectionManager projectionManager;
        private readonly ITokenizer tokenizer;
    
        /// <summary>
        /// Initializes a new instance of the <see cref="TaskSchedulerService" /> class.
        /// </summary>
        /// <param name="queryRepository">The query repository.</param>
        /// <param name="contentManager">The content manager.</param>
        /// <param name="projectionManager">The projection manager.</param>
        /// <param name="tokenizer">The tokenizer.</param>
        public TaskSchedulerService(IRepository<QueryPartRecord> queryRepository, IContentManager contentManager, IProjectionManager projectionManager, ITokenizer tokenizer)
        {
            this.tokenizer = tokenizer;
            this.projectionManager = projectionManager;
            this.contentManager = contentManager;
            this.queryRepository = queryRepository;

            T = NullLocalizer.Instance;
        }

        public Localizer T { get; set; }

        /// <summary>
        /// Gets the content.
        /// </summary>
        /// <param name="queryId">The query id.</param>
        /// <param name="timeFrame">The time frame.</param>
        /// <returns></returns>
        public IEnumerable<ContentItem> GetContent(int queryId, TimeFrame timeFrame = null)
        {
            var query = Query(queryId);

            if (timeFrame == null) 
            {
                return query.Slice(0, 0);
            }

            Action<IAliasFactory> alias = x => x.ContentPartRecord<KendoTaskPartRecord>();
            //check if the start time is within the timeframe 
            Action<IHqlExpressionFactory> overrunsTheStartDate = x => x.Between("StartUtc", timeFrame.StartUtc, timeFrame.EndUtc);
            //check if the end time is within the timeframe
            Action<IHqlExpressionFactory> overrunsTheEndDate = x => x.Between("EndUtc", timeFrame.StartUtc, timeFrame.EndUtc);

            Action<IHqlExpressionFactory> isWithinOrRunsOverStartOrEndDates = x => x.Or(overrunsTheStartDate, overrunsTheEndDate);

            query = query.Where(alias, isWithinOrRunsOverStartOrEndDates);

            return query.Slice(0, 0);
        }

        /// <summary>
        ///Gets the total count of items from the specified query id.
        /// </summary>
        /// <param name="queryId">The query id.</param>
        /// <returns></returns>
        public int Total(int queryId)
        {
            return this.projectionManager.GetCount(queryId);
        }

        private IHqlQuery Query(int queryId)
        {
            var queryRecord = queryRepository.Get(queryId);

            if (queryRecord == null)
            {
                throw new ArgumentException("queryId");
            }

            var query = contentManager.HqlQuery().ForVersion(VersionOptions.Published);

            this.FilterFromQueryRecord(query, queryRecord);

            return query;
        }

        private void FilterFromQueryRecord(IHqlQuery contentQuery, QueryPartRecord queryPartRecord) 
        {
            var allAvailableFilters = this.projectionManager.DescribeFilters().ToList();
            var allAvailableSortCriteria = this.projectionManager.DescribeSortCriteria().ToList();

            foreach (var group in queryPartRecord.FilterGroups)
            {
                FilterFromGroup(contentQuery, group, allAvailableFilters);
                SortFromGroup(contentQuery, group, allAvailableSortCriteria);
            }
        }

        private void FilterFromGroup(IHqlQuery contentQuery, FilterGroupRecord group, IEnumerable<TypeDescriptor<FilterDescriptor>> availableFilters) 
        {
            foreach (var filter in group.Filters)
            {
                var tokenizedState = tokenizer.Replace(filter.State, new Dictionary<string, object>());
                var filterContext = new FilterContext
                {
                    Query = contentQuery,
                    State = FormParametersHelper.ToDynamic(tokenizedState)
                };

                var desriptor = availableFilters.SelectMany(x => x.Descriptors).FirstOrDefault(x => x.Category == filter.Category && x.Type == filter.Type);

                if (desriptor == null)
                    continue;

                desriptor.Filter(filterContext);

                contentQuery = filterContext.Query;
            }
        }

        private void SortFromGroup(IHqlQuery contentQuery, FilterGroupRecord group, IEnumerable<TypeDescriptor<SortCriterionDescriptor>> availableSortCriteria) 
        {
            foreach (var sortCriterion in group.QueryPartRecord.SortCriteria)
            {
                var sortCriterionContext = new SortCriterionContext
                {
                    Query = contentQuery,
                    State = FormParametersHelper.ToDynamic(sortCriterion.State)
                };

                string category = sortCriterion.Category;
                string type = sortCriterion.Type;

                // look for the specific filter component
                var descriptor = availableSortCriteria
                    .SelectMany(x => x.Descriptors)
                    .FirstOrDefault(x => x.Category == category && x.Type == type);

                // ignore unfound descriptors
                if (descriptor == null)
                {
                    continue;
                }

                // apply alteration
                descriptor.Sort(sortCriterionContext);

                contentQuery = sortCriterionContext.Query;
            }
        }

        public void Delete(int id)
        {
            var item = contentManager.Get(id);

            if (item == null) 
            {
                //lets presume it could have already been done.
                return;
            }

            contentManager.Remove(item);
        }

        public void Create(ViewModels.SchedulerTaskViewModel viewModel, string contentType = null, bool publish = true)
        {
            //scheduler needs to know the content type which to create the content item to.
            //override this service if you already have as content type to use or overload the action with the content type 

            var definedContentType = string.IsNullOrWhiteSpace(contentType) ? DefinitionContentType.KendoSchedulerTaskType : contentType;
            var model = contentManager.New(definedContentType);

            this.UpdateDataModelFromViewModel(model, viewModel);
            this.contentManager.Create(model);

            if (publish) 
            { 
                contentManager.Publish(model);
            }

            viewModel.TaskId = model.Id;
        }

        public void Update(ViewModels.SchedulerTaskViewModel viewModel)
        {
            var model = contentManager.Get(viewModel.TaskId);

            if (model == null) 
            {
                return;
            }

            this.UpdateDataModelFromViewModel(model, viewModel);
        }

        private void UpdateDataModelFromViewModel(ContentItem model, ViewModels.SchedulerTaskViewModel viewModel) 
        {
            var titlePart = model.As<TitlePart>();
            var taskPart = model.As<KendoTaskPart>();

            titlePart.Title = viewModel.Title;

            taskPart.Description = viewModel.Description;
            taskPart.StartUtc = viewModel.Start;
            taskPart.EndUtc = viewModel.End;

            taskPart.RecurrenceException = viewModel.RecurrenceException;
            taskPart.RecurrenceId = viewModel.RecurrenceId;
            taskPart.RecurrenceRule = viewModel.RecurrenceRule;
        }
    }
}